`timescale 1ns / 1ps

//******************************************************************************
//* 4-digit seven-segment display driver module.                               *
//******************************************************************************
module SevenSegmentDisplay(
   //Clock and reset.
   input  wire       clk,     //16 MHz system clock
   input  wire       rst,     //Reset signal
   
   //Input data.
   input  wire [3:0] dig0,    //Value displayed on DIG0
   input  wire [3:0] dig1,    //Value displayed on DIG1
   input  wire [3:0] dig2,    //Value displayed on DIG2
   input  wire [3:0] dig3,    //Value displayed on DIG3
   input  wire [3:0] dp,      //Decimal points
   
   //Signals driving the seven-segment display.
   output wire [7:0] seg_n,   //Segment control signal (active-low)
   output wire [3:0] dig_n    //Digit select signal (active-low)
);

//******************************************************************************
//* Generating the 250 Hz enable signal for the ring counter.                  *
//******************************************************************************
reg  [15:0] clk_divider;
wire        clk_divider_tc = (clk_divider == 16'd0);

always @(posedge clk)
begin
   if (rst || clk_divider_tc)
      clk_divider <= 16'd63999;
   else
      clk_divider <= clk_divider - 16'd1;
end


//******************************************************************************
//* Shift register based ring counter that generates the digit enable signals. *
//******************************************************************************
reg [3:0] dig_shr;

always @(posedge clk)
begin
   if (rst)
      dig_shr <= 4'b0001;
   else
      if (clk_divider_tc)
         dig_shr <= {dig_shr[2:0], dig_shr[3]};
end

//Driving the digit select signal.
assign dig_n = ~dig_shr;


//******************************************************************************
//* Input data select multiplexer.                                             *
//******************************************************************************
wire [3:0] dig_mux = (dig0 & {4{dig_shr[0]}}) | 
                     (dig1 & {4{dig_shr[1]}}) |
                     (dig2 & {4{dig_shr[2]}}) |
                     (dig3 & {4{dig_shr[3]}});
                     
wire dp_mux = (dp[0] & dig_shr[0]) |
              (dp[1] & dig_shr[1]) |
              (dp[2] & dig_shr[2]) |
              (dp[3] & dig_shr[3]);
                     

//******************************************************************************
//* Hexadecimal to seven-segment decoder.                                      *
//******************************************************************************
wire [7:0] seg_data;

HexTo7Seg HexTo7Seg(.value(dig_mux), .code(seg_data));

//Driving the segment control signal.
assign seg_n = ~{dp_mux, seg_data[6:0]};


endmodule
